@@ -1,8 +1,9 @@
 package Class::PObject::Iterator;
 
-# $Id: Iterator.pm,v 1.5 2003/08/28 16:32:24 sherzodr Exp $
+# $Id: Iterator.pm,v 1.5.2.2 2003/09/06 10:14:56 sherzodr Exp $
 
 use strict;
+#use diagnostics;
 use Carp 'croak';
 use vars ('$VERSION');
 
@@ -15,7 +16,7 @@ sub new {
     my $self = {
         pobject     => $_[0],
         data_set    => $_[1],
-        next_pointer=> 0,
+        next_ptr=> 0,
         last_ptr    =>  scalar(@{$_[1]})-1
     };
 
@@ -41,7 +42,6 @@ sub next {
 
 sub size {
     my $self = shift;
-
     return $self->{last_ptr} - $self->{next_ptr} + 1
 }
 
@@ -82,11 +82,8 @@ sub dump {
     return $d->Dump
 }
 
-
-
 1;
 __END__
-# Below is stub documentation for your module. You'd better edit it!
 
 =head1 NAME
 
@@ -154,20 +151,12 @@ and want to start from clean data set:
 
 =back
 
-
 =head1 SEE ALSO
 
 L<Class::PObject>
 
-=head1 AUTHOR
-
-Sherzod B. Ruzmetov, E<lt>sherzodr@cpan.orgE<gt>
-
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2003 by Sherzod B. Ruzmetov.
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+For author and copyright information refer to Class::PObject's L<online manual|Class::PObject>.
 
 =cut
