@@ -3,6 +3,7 @@ package Class::PObject::Test::HAS_A;
 use strict;
 #use diagnostics;
 use Test::More;
+use Data::Dumper;
 use vars ('$VERSION', '@ISA');
 
 BEGIN {
@@ -108,6 +109,8 @@ sub run {
     ok($author->name eq "Sherzod Ruzmetov",    $article->author->name);
     ok(ref($author) eq "PO::Author",                ref($article->author));
 
+    print Dumper($article->columns);
+
     ok(PO::Article->count({author=>$author}) == 1);
     ok(PO::Article->remove_all);
     ok(PO::Article->count({author=>$author}) == 0);
