@@ -1,11 +1,18 @@
 package Class::PObject::Type::TEXT;
 
-# $Id: TEXT.pm,v 1.1.2.3 2003/09/06 10:15:00 sherzodr Exp $
+# $Id: TEXT.pm,v 1.1.2.4 2003/09/06 14:00:51 sherzodr Exp $
 
 use strict;
 #use diagnostics;
 use vars ('$VERSION', '@ISA');
 use Class::PObject::Type;
+use overload (
+    '""' => sub { $_[0]->id },
+    bool => sub { $_[0]->id ? 1 : 0 },
+    fallback => 1,
+);
+
+
 @ISA = ("Class::PObject::Type");
 
 $VERSION = '1.01';
